
# TODO generalize this somehow, probably using AllSpin/AllPol as defaults
"""
    parse_process(string::AbstractString, model::QEDModel)

Parse a string representation of a process, such as "ke->ke" into the corresponding [`QEDProcessDescription`](@ref).
"""
function parse_process(
    str::AbstractString,
    model::QEDModel,
    inphpol::AbstractDefinitePolarization,
    inelspin::AbstractDefiniteSpin,
    outphpol::AbstractDefinitePolarization,
    outelspin::AbstractDefiniteSpin,
)
    inParticles = Dict(
        ParticleStateful{Incoming, Photon, SFourMomentum} => 0,
        ParticleStateful{Incoming, Electron, SFourMomentum} => 0,
        ParticleStateful{Incoming, Positron, SFourMomentum} => 0,
    )
    outParticles = Dict(
        ParticleStateful{Outgoing, Photon, SFourMomentum} => 0,
        ParticleStateful{Outgoing, Electron, SFourMomentum} => 0,
        ParticleStateful{Outgoing, Positron, SFourMomentum} => 0,
    )

    if !(contains(str, "->"))
        throw("Did not find -> while parsing process \"$str\"")
    end

    (inStr, outStr) = split(str, "->")

    if (isempty(inStr) || isempty(outStr))
        throw("Process (\"$str\") input or output part is empty!")
    end

    for t in types(model)
        if (is_incoming(t))
            inCount = count(x -> x == String(t)[1], inStr)

            if inCount != 0
                inParticles[t] = inCount
            end
        end
        if (is_outgoing(t))
            outCount = count(x -> x == String(t)[1], outStr)
            if outCount != 0
                outParticles[t] = outCount
            end
        end
    end

    if length(inStr) != sum(values(inParticles))
        throw("Encountered unknown characters in the input part of process \"$str\"")
    elseif length(outStr) != sum(values(outParticles))
        throw("Encountered unknown characters in the output part of process \"$str\"")
    end

    in_ph = inParticles[ParticleStateful{Incoming, Photon, SFourMomentum}]
    in_el = inParticles[ParticleStateful{Incoming, Electron, SFourMomentum}]
    in_pos = inParticles[ParticleStateful{Incoming, Positron, SFourMomentum}]
    out_ph = outParticles[ParticleStateful{Outgoing, Photon, SFourMomentum}]
    out_el = outParticles[ParticleStateful{Outgoing, Electron, SFourMomentum}]
    out_pos = outParticles[ParticleStateful{Outgoing, Positron, SFourMomentum}]
    in_spin_pols = tuple([i <= in_ph ? inphpol : inelspin for i in 1:(in_ph + in_el)]...)
    out_spin_pols = tuple([i <= out_ph ? outphpol : outelspin for i in 1:(out_ph + out_el)]...)

    return GenericQEDProcess(in_ph, out_ph, in_el, out_el, in_pos, out_pos, in_spin_pols, out_spin_pols)
end
